#!/usr/bin/python3
# -*- coding: utf-8 -*-
import hashlib
import requests
import random
import re
import fnmatch
import subprocess
import linecache
import sys
import difflib
import decimal
from json import load, loads, dump
from os import environ, listdir, walk, path
from typing import List, Dict, Tuple, Set
from datetime import datetime
from lockfile import locked
from statistics import mean
from time import sleep
import erebo.settings.settings as settings


def clean_text(text_to_covert: str) -> str:
        """Function 'get_base_regex_expression'.
            Return regular expresion to find in a text

        Note:
            Do not include the `self` parameter in the ``Args`` section.

        Args:
            text_to_analize (:obj:`str`):  text to analyze.

        Returns:
            :obj:`str` successful, :obj:`None`  otherwise.

        """
        
        text_to_covert = text_to_covert.lower().strip()
        regex_spacechars = (
            ';',
            '-',
            ',',
            '\n',
            ':',
            '_'
        )
        for regex_char in regex_spacechars:
            text_to_covert = text_to_covert.replace(regex_char, ' ')
        
        regex_metachars = (
            '\\\\',
            '.',
            '^',
            ']',
            '[',
            '$',
            '|',
            '?',
            '*',
            '{',
            '}',
            '(',
            ')',
            '/',
            '!',
        )
        for regex_char in regex_metachars:
            text_to_covert = text_to_covert.replace(regex_char, ' ')
        
        special_chars = (
            ('á','a'),
            ('é','e'),
            ('í','i'),
            ('ó','o'),
            ('ú','u'),
            ('à','a'),
            ('è','e'),
            ('ì','i'),
            ('ò','o'),
            ('ù','u'),
            ('ñ','n'),
        )
        for special_char in special_chars:
            text_to_covert = text_to_covert.replace(
                special_char[0], special_char[1])
        text_to_covert = ' '.join([
            wr for wr in text_to_covert.split(' ') 
            if wr != ''])
        return text_to_covert.replace(' +',' ')

def ping_vpn(proxy: str = None):
    proxies = None
    if proxy:
        proxies = {
            'http': proxy,
            'https': proxy,
        }
    ret = {"code": 200, "response": {}}
    try:
        url = 'http://ifconfig.co/json'
        ret['response'] = loads(requests.request(
            'GET',
            url,
            proxies=proxies,
            timeout=15
        ).text)
    except Exception:
        ret['code'] = 400
    return ret

def generate_hash(text):
    aux = hashlib.sha224()
    aux.update(text.encode('utf-8'))
    return aux.hexdigest()

def get_enviroment():
    # enviroments = load(open('erebo/config/enviroment.json'))
    enviroments = load_enviroment()
    sc_enviroment = environ.get('sc_enviroment', 'media')
    file_name = enviroments.get(sc_enviroment, 'media.ini')
    ret = {'file_name': file_name, 'sc_enviroment': sc_enviroment}
    return ret

def copy_file(dirpath: str, file:str, container_name: str, 
        container_dirpath: str) -> bool:
    ret = False
    if dirpath[-1] == '/':
        dirpath = dirpath[:-1]
    if container_dirpath[-1] == '/':
        container_dirpath = container_dirpath[:-1]
    container_path = ':'.join((container_name, container_dirpath))
    filepath = '/'.join((dirpath, file))
    cmd = ['docker', 'cp', filepath, container_path ]
    try:
        subprocess.check_call(cmd)
        ret = True
    except subprocess.CalledProcessError as e:
        print(" {} | Error coping auth  Error: {}".format(e, container_name))
    return ret

def to_route(src_ip: str, to_ip: str, jump: int, add: bool,
        every_jump: int = 1) -> bool:
    # every_jump make x connection continues in the same ip
    # this work to get the external ip in te connection
    ret = False
    action = 'A' if add else 'D'
    jump = int(jump)
    # multiply jump for each every_jump to allow connecting to the same ip fox every_jump times
    jump *= every_jump
    for x in range(every_jump):
        jump -= x
        str_cmd = '''
            iptables 
            -t nat 
            -{} 
            PREROUTING 
            -s {} 
            -i eth0 
            -p tcp 
            --dport 80 
            -m statistic 
            --mode nth 
            --every {} 
            --packet 0 
            -j DNAT 
            --to-destination {}:3128'''.format(action, src_ip, jump, to_ip)
        try:
            subprocess.check_call(str_cmd.split())
            ret = True
        except subprocess.CalledProcessError as e:
            print("Error {}".format(e))
    return ret

def deny_all(add: bool) -> bool:
    ret = False
    action = 'A' if add else 'D'
    str_cmd = '''
        iptables 
        -{} 
        INPUT 
        -p tcp 
        -i eth0 
        --dport 80 
        -j REJECT'''.format(action)
    try:
        subprocess.check_call(str_cmd.split())
        ret = True
    except subprocess.CalledProcessError as e:
        print("Error {}".format(e))
    return ret

def route_target(add: bool) -> bool:
    ret = False
    action = 'A' if add else 'D'
    # str_cmd = '''
    #     iptables 
    #     -t nat 
    #     -{} 
    #     POSTROUTING 
    #     -j MASQUERADE'''.format(action)
    str_cmd = '''
        iptables 
        -t nat 
        -{} 
        POSTROUTING -p tcp
        -j MASQUERADE'''.format(action)
    try:
        subprocess.check_call(str_cmd.split())
        ret = True
    except subprocess.CalledProcessError as e:
        print("Error {}".format(e))
    return ret

def get_location_from_file(vpn_name: str, file_name: str):
    # first group is to Country
    # second group is to location if exist
    country = ''
    info = ''
    files_auth = load_vpns_config()
    regex_compiles = files_auth[vpn_name]['regex_location']
    for regex_compile in regex_compiles:
        cp = re.compile(regex_compile, re.IGNORECASE)
        locations = re.match(cp, file_name)
        try:
            country = clean_text(locations.group(1))
        except Exception:
            pass
        try:
            info = clean_text(locations.group(2))
        except Exception:
            pass

        if country:
            break
    ret = {
        "country": country,
        "info": info
    }
    return ret

def print_message(message: str):
    print('{1}  {0}'.format(
        message,
        datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S')))

def print_exception():
    exc_type, exc_obj, tb = sys.exc_info()
    f = tb.tb_frame
    lineno = tb.tb_lineno
    filename = f.f_code.co_filename
    linecache.checkcache(filename)
    line = linecache.getline(filename, lineno, f.f_globals)
    ret = {
        'type': "exception",
        'file_name': filename,
        'line': lineno,
        'expresion': line.strip(),
        'error': exc_obj
    }
    return ret

def alias_acuraccy(text_src: str, text_compare: str) -> List:
        ret = []
        only_title = 0
        text_clean = tuple(clean_text(text_compare).split(' '))
        alias = text_src.split('###')
        acum_acc = []
        for title in alias:
            accuracy = 0
            clean_alias = clean_text(title)                                
            title_split = tuple(clean_alias.split(' '))
            tmp_list = []                    
            for row in text_clean:
                tmp_list.append((
                    row, 
                    difflib.SequenceMatcher(
                        None, title_split[0], row).ratio()))
            first_word = max(tmp_list, key=lambda x: x[1])[0]
            first_word_position = text_clean.index(first_word)
            cant_words = len(title_split)
            compare_words = ' '.join(
                text_clean[first_word_position:first_word_position + cant_words])
            accuracy = difflib.SequenceMatcher(
                None, clean_alias, compare_words).ratio()
            acum_acc.append(accuracy)
        acc_mean = mean(acum_acc)
        if len(acum_acc) == 1:
            only_title = 1
        ret.append({
            "alias": text_src,
            "accuracy": decimal.Decimal(round(acc_mean,3)).quantize(
                decimal.Decimal('0.000'), rounding=decimal.ROUND_UP),
            "only_title": only_title
        })
        return ret

def get_external_ip(container_name, retry: int = 15) -> str:
    cnt = 1
    result = ''
    while True:
        response = ping_vpn()
        if response['code'] == 200 and response['response']:
            result = response['response']['ip']
            break
        sleep(0.3)
        cnt += 1
        if cnt > retry:
            print_message('| {} | \tStep: Error getting VPN external IP  Error: 1005'.format(container_name))
            break
    return result

def get_vpn_external_ip(container: Dict, external_ip: str, retry: int = 35) -> Dict:
    result = ''
    cnt = 1
    while True:
        response = ping_vpn(container["ip_addrs"]+':'+container["port"])
        if (response['code'] == 200 and response['response'] and 
                response['response']['ip'] != external_ip):
            result = response['response']
            break
        sleep(0.3)
        cnt += 1
        if cnt > retry:
            print_message('| {} | \tStep: Error getting VPN external IP  Error: 1005'.format(container['name'])
            )
            break
    return result

def load_vpns_config(file_location: str = None) -> Dict:
    try:
        if not file_location:
            file_location = f'{settings.APP_PATH}/config/files_auth.json'
        ret = load(open(file_location))
    except FileNotFoundError:
        print_message('| {} | \tStep: Error Open file {}'.format('', file_location))
    finally:
        return ret

def load_countries(file_location: str = None) -> Dict:
    try:
        if not file_location:
            file_location = f'{settings.APP_PATH}/config/countries.json'
        ret = load(open(file_location, 'r'))
    except FileNotFoundError:
        print_message('| {} | \tStep: Error Open file {}'.format('', file_location))
    finally:
        return ret

def load_enviroment(file_location: str = None) -> Dict:
    try:
        if not file_location:
            file_location = f'{settings.APP_PATH}/config/enviroment.json'
        ret = load(open(file_location, 'r'))
    except FileNotFoundError:
        print_message('| {} | \tStep: Error Open file {}'.format('', file_location))
    finally:
        return ret


#!/usr/bin/python3
# -*- coding: utf-8 -*-
import docker
import time
import hashlib
import os
import signal
from typing import List, Dict, Tuple, Set 
from docker.types import LogConfig
from time import sleep
import erebo.core.utils as utils
import erebo.core.vpn_utils as vutils
import erebo.settings.settings as settings
from erebo.model.rediss import RedisWrapper



class DockerProject(object):

    def __init__(self):
        # self.client = docker.from_env()
        # self.client = docker.DockerClient(version='1.24')
        self.client = docker.DockerClient()
        self.containers = list(dict())
        self.logs_check = {}

    def container_run(
            self, image: str, name: str, command: str, volumes: list = [],
            devices: list = [], cap_add: list = [], dns: list = [],
            detach: bool = True, tty: bool = True, stdout: bool = True,
            stderr: bool = True, log_config = {}, environment = [], ports: Dict = {}):
        """[summary]
        
        Arguments:
            image {str} -- [description]
            name {str} -- [description]
            command {str} -- [description]
        
        Keyword Arguments:
            volumes {list} -- [description] (default: {[]})
            devices {list} -- [description] (default: {[]})
            cap_add {list} -- [description] (default: {[]})
            dns {list} -- [description] (default: {[]})
            detach {bool} -- [description] (default: {True})
            tty {bool} -- [description] (default: {True})
            stdout {bool} -- [description] (default: {True})
            stderr {bool} -- [description] (default: {True})
            log_config {dict} -- [description] (default: {{}})
            environment {list} -- [description] (default: {[]})
            port {dict} -- [description] (default: {{}})
        
        Returns:
            [type] -- [description]
        """


        #log_config format:
        # {"max_size": "1g", "labels": "erebo"}
        volumes_dicts = dict()
        if log_config:
            lc = LogConfig(
                type=LogConfig.types.JSON,
                config=log_config
            )
        else:
            lc = LogConfig(
                type=LogConfig.types.JSON,
                config={"max-size": "1g", "labels": "erebo"}
            )
        for volume in volumes:
            vol_host, vol_container, vol_permission = volume.split(':')
            volumes_dicts[vol_host] = {
                'bind': vol_container,
                'mode': vol_permission
            }
        container = None
        restart_policy = {
            "Name": "unless-stopped",
            "MaximumRetrycount": 0,
        }
        container = self.client.containers.run(
            image=image, name=name, detach=detach, tty=tty, stdout=stdout,
            stderr=stderr, volumes=volumes_dicts, devices=devices,
            cap_add=cap_add, dns=dns, command=command,
            restart_policy=restart_policy, environment=environment,
            log_config=lc, ports=ports)
        container_dict = {
            "name": name,
            "container": container,
            "status": "run",
            "created_at": time.time()
        }
        self.containers.append(container_dict)
        return container

    def container_exec(
            self, name: str, command: str, stdout: bool = True,
            stderr: bool = True, stdin: bool = False, tty: bool = False,
            detach: bool = True):
        """[summary]
        
        Arguments:
            name {str} -- [Container name]
            command {str} -- [CMD to execute]
        
        Keyword Arguments:
            stdout {bool} -- [Attach to stdout] (default: {True})
            stderr {bool} -- [Attach to stderr] (default: {True})
            stdin {bool} -- [Attach to stdin] (default: {False})
            tty {bool} -- [Allocate a pseudo-TTY.] (default: {False})
            detach {bool} -- [If true, detach from the exec command] (default: {True})
        """
        con = self.get_container(name=name)
        run_cmd = con.exec_run(cmd=command, detach=detach)
        return run_cmd

    def container_logs(self, name) -> bool:
        container = self.get_container(name)
        if container:
            return container.logs()
        return False

    def container_wait(self, name) -> bool:
        container = self.get_container(name)
        if container:
            container.wait()
            return True
        return False

    def container_stop(self, name) -> bool:
        container = self.get_container(name)
        if container:
            try:
                container.stop()
                return True
            except Exception:
                return False
        return False

    def container_start(self, name):
        container = self.get_container(name)
        if container:
            try:
                container.start()
                return True
            except Exception:
                return False
        return False

    def container_remove(self, name):
        container = self.get_container(name)
        if container:
            try:
                container.remove()
                return True
            except Exception:
                return False
        return False

    def container_stop_remove(self, name):
        container = self.get_container(name)
        if container:
            if (self.container_stop(name) and
               self.container_remove(name)):
                return True
        return False

    def get_container(self, name):
        try:
            return [
                x
                for x in self.get_containers()
                if x.name == name][0]
        except Exception as e:
            utils.print_message('| {} | ERROR getting container Err:{}'.format(name, e))

    def get_containers(self, all: bool = True):
        try:
            return self.client.containers.list(all)
        except Exception:
            return None

    def remove_container_name(self, name):
        try:
            for x in self.get_containers():
                if x.name == name:
                    x.remove(force=True)
                    return True
            return False
        except Exception as e:
            utils.print_message('| {} | ERROR removing container next name Err:{}'.format(name, e))
            return None

    def get_container_nextname(self, prefix_name:str) -> str:
        """[summary]
        
        Arguments:
            prefix_name {str} -- [description]
        
        Returns:
            str -- [description]
        """
        try:
            nextdocker = 1
            if self.get_containers() is None:
                print("Create new:" + prefix_name + '_' + str(nextdocker))
                return prefix_name + '_' + str(nextdocker)
            for x in self.get_containers():
                if prefix_name in x.name:
                    if x.status == 'exited':
                        print("Remove and reuse:" + x.name)
                        x.remove()
                        return x.name
                    else:
                        data = x.name.split('_')
                        if 'openvpn' == data[0]:
                            if int(data[2]) >= nextdocker:
                                nextdocker = int(data[2]) + 1
                        # elif 'youtube' == data[0]:
                        #     if int(data[2]) >= nextdocker:
                        #         nextdocker = int(data[2]) + 1
                        else:
                            if int(data[1]) >= nextdocker:
                                nextdocker = int(data[1]) + 1
            print("Create new:" + prefix_name + '_' + str(nextdocker))
            return prefix_name + '_' + str(nextdocker)
        except Exception as e:
            utils.print_message('| {} | ERROR getting container next name Err:{}'.format(prefix_name, e))
            return None

    def get_container_process(self, name: str) -> List:
        container = self.get_container(name=name)
        process = [x for x in container.top()['Processes']]
        return process

    def get_container_process_per_name(self, name:str, proc_name:str) -> List:
        process = self.get_container_process(name)
        proc_list = [x for x in process if proc_name in x[7]]
        return proc_list

    def get_container_info(self, container_name: str) -> Dict:
        result = {}
        try:
            container = self.get_container(name=container_name)
            result = {
                "container": container,
                "name": container.name, 
                "status": container.status, 
                "ip_addrs": container.attrs['NetworkSettings']['IPAddress'],
                "ports": container.attrs['NetworkSettings']['Ports'],
                "port": list(
                    container.attrs['NetworkSettings']['Ports'])[0].split('/')[0]
            }
        except Exception as e:
            utils.print_message('| {} | ERROR getting container info Err:{}'.format(container_name, e))
        finally:
            return result

    def kill_process(self, pids: List, container_name: str) -> bool:
        ret = False
        try:
            for pid in pids:
                utils.print_message('| {} | \tStep: kill PID {}'.format(
                    container_name, pid))
                os.kill(int(pid), signal.SIGKILL)
            ret = True
        except Exception as e:
            utils.print_message(
                '| {} | ERROR killing process Err:{}'.format(pids, e))
        return ret

    def kill_process_sudo(self, pids: List, container_name: str) -> bool:
        ret = False
        command = 'kill -9'
        sudoPassword = settings.settings['sudo_pass']
        try:
            for pid in pids:
                utils.print_message('| {} | \tStep: kill PID {}'.format(
                    container_name, pid))
                os.system(
                    f'echo {str(sudoPassword)}|sudo -S {str(command)} {int(pid)}'
                    )
            ret = True
        except Exception as e:
            utils.print_message(
                '| {} | ERROR killing process Err:{}'.format(pids, e))
        return ret
        

    def exec_vpn(self, container_name:str, iso2: str = None) -> Dict:
        ret = {}
        vpn_name = vutils.get_vpn_name()
        iso2_error = "There are no vpn files from '{}'".format(iso2)
        try:
            auth_file = vutils.get_auth_available(vpn_name)
            if auth_file['full_path']:
                if iso2:
                    ovpn_file = vutils.get_ovpn_filename_iso(auth_file, iso2)
                else:
                    ovpn_file = vutils.get_ovpn_filename(auth_file['directory'])
                replace_auth = vutils.replace_auth_user(
                    ovpn_file['full_path'], auth_file['full_path'])
                if replace_auth:
                    utils.print_message('| {} |\tStep: Saving VPN Info'.format(container_name))
                    with RedisWrapper() as db:
                        db.add_container_auth(container_name=container_name,
                            auth_file=auth_file['file'],
                            vpn_name=auth_file['vpn_name'],
                            vpn_file=ovpn_file['full_path'])
                    if auth_file['container_dirpath'][-1] == '/':
                        auth_file['container_dirpath'] = auth_file['container_dirpath'][:-1]
                    cp_ovpn = utils.copy_file(
                        ovpn_file['dirpath'],
                        ovpn_file['filename'],
                        container_name,
                        auth_file['container_dirpath']
                    )
                    cp_auth = utils.copy_file(
                        auth_file['directory'],
                        auth_file['file'],
                        container_name,
                        auth_file['container_dirpath']
                    )
                    if cp_ovpn and cp_auth:
                        file_name = '/'.join((
                            auth_file['container_dirpath'],
                            ovpn_file['filename'].replace(' ', '\ ')))
                        cmd = "openvpn --config"
                        cmd_exec = ' '.join((cmd, file_name))
                        _ = self.container_exec(container_name, cmd_exec, True)
                        ret = {
                            "container_name": container_name,
                            "auth_file": auth_file['file'],
                            "vpn_name": auth_file['vpn_name'],
                            "vpn_file": ovpn_file['full_path']
                        }
                    else:
                        utils.print_message('| {} | ERROR coping auth and ovpn file Err:{}'.format(container_name, '1001'))
                else:
                    utils.print_message('| {} | ERROR Error in auth file Err:{}'.format(container_name, '1001'))
            else:
                if auth_file:
                    vutils.decrease_auth(vpn_name, auth_file['file'])
                    with RedisWrapper() as db:
                        db.del_container_auth(container_name=container_name)
                utils.print_message('| {} | ERROR There are no free ovpn, all the authentication files have been used Err:{}'.format(container_name, '1001'))
        except Exception as e:
            utils.print_message('| {} | ERROR executing vpn process Err:{}'.format(container_name, e))
            if iso2:
                utils.print_message('| {} | ERROR {}'.format(container_name, iso2_error))
        finally:
            return ret


#!/usr/bin/python3
# -*- coding: utf-8 -*-
from typing import List, Dict, Tuple, Set
from time import sleep
import os
import signal
from erebo.core.docker_project import DockerProject
from erebo.controllers.check_vpn import CheckVpn
from erebo.model.rediss import RedisWrapper
import erebo.core.utils as utils
import erebo.core.utils as utils
import erebo.core.vpn_utils as vutils

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"


class RestartVpn(object):

    def __init__(self, *args, **kwargs):
        self.__docker_client = DockerProject()
        self.external_ip = ''
        self.vpn_exgternal_ip = {}
        # return super().__init__(*args, **kwargs)

    def _kill(self, container_name: str, process_name) -> bool:
        ret = False
        try:
            process = self.__docker_client.get_container_process_per_name(
                container_name, process_name)
            pids = [x[1] for x in process]
            # is_kill = self.__docker_client.kill_process(
            #     pids=pids, container_name=container_name)
            is_kill = self.__docker_client.kill_process_sudo(
                pids=pids, container_name=container_name)
            ret = is_kill
        except Exception:
            print("_kill {}".format(utils.print_exception()))
        finally:
            return ret

    def _init_vpn(self, container_name:str, iso2:str = None) -> Dict:
        ret = {}
        self.external_ip = utils.get_external_ip(container_name=container_name)
        try:
            vpn = self.__docker_client.exec_vpn(container_name, iso2)
            container = self.__docker_client.get_container_info(container_name)
            self.vpn_exgternal_ip = utils.get_vpn_external_ip(
                    container=container, external_ip=self.external_ip)
            if self.vpn_exgternal_ip:
                ret = vpn
        except Exception:
            print("_init_vpn {}".format(utils.print_exception()))
        finally:
            return ret

    def run(self, container_name:str, iso2:str = None) -> bool:
        utils.print_message('| {} | \tStep: Restart VPN'.format(container_name))
        ret = False
        try:
            with RedisWrapper() as db:
                # get auth_file
                utils.print_message('| {} | \tStep: Getting AUTH File'.format(container_name))
                auth_file = db.get_container_auth(container_name=container_name)
                # kill vpn
                utils.print_message('| {} | \tStep: Killing VPN'.format(container_name))
                if self._kill(
                        container_name=container_name,
                        process_name='openvpn'):
                    # decrement used auth file
                    utils.print_message('| {} | \tStep: Decreseing AUTH'.format(container_name))
                    # delete vpn and auth row in redis
                    utils.print_message('| {} |\tStep: Deleting AUTH'.format(container_name))
                    if db.del_container_auth(
                            container_name=container_name):
                        # up vpn
                        utils.print_message('| {} | \tStep: Start VPN'.format(container_name))
                        vpn = self._init_vpn(container_name=container_name,  iso2=iso2)
                        if vpn:
                            # add vpn and auth row in redis
                            utils.print_message('| {} | \tStep: Adding AUTH'.format(container_name))
                            db.add_container_auth(
                                container_name=vpn['container_name'],
                                auth_file=vpn['auth_file'], 
                                vpn_name=vpn['vpn_name'],
                                vpn_file=vpn['vpn_file'])
                            utils.print_message('| {} | \tStep: Adding Container'.format(container_name))
                            db.add_container_vp_ip(
                                container_name=vpn['container_name'],
                                value=self.vpn_exgternal_ip)
                            ret = True
        except Exception:
            print("run {}".format(utils.print_exception()))
        finally:
            return ret


#!/usr/bin/python3
# -*- coding: utf-8 -*-
from typing import List, Dict, Tuple, Set
from random import randrange
from time import sleep
from erebo.core.docker_project import DockerProject
from erebo.model.rediss import RedisWrapper
import erebo.settings.settings as settings
import erebo.core.utils as utils
import erebo.core.vpn_utils as vutils


__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"

class UpContainer(object):
    """[summary]
    
    Arguments:
        object {[type]} -- [description]
    """

    def __init__(self, *args, **kwargs):
        self._dc = DockerProject()

    def create_container(self, name: str, iso2: str = None) -> bool:
        ret = False
        utils.print_message('| {} | \tStep: Getting External IP'.format(name))
        external_ip = utils.get_external_ip(container_name=name)
        try:
            image = "tsari/openvpn"
            port = randrange(5000, 7999)
            local_port = '3128'
            utils.print_message('| {} | \tStep: Creating Container'.format(name))
            ports = dict({ "".join((str(local_port), '/tcp')): port})
            container = self._dc.container_run(
                image, 
                name,
                '',
                ['/opt/vpn:/opt/vpn:rw'],
                ['/dev/net/tun'],
                ['NET_ADMIN'],
                ['8.8.8.8','1.1.23.1'],
                True,
                True,
                True,
                True,
                {},
                [],
                ports)
            utils.print_message('| {} | \tStep: Starting VPN'.format(name))
            if container:
                cnt = self._dc.get_container(name)
                cnt_ip = cnt.attrs['NetworkSettings']['Networks']['bridge']['IPAddress']
                host_ip = utils.get_external_ip(container_name=name)
                with RedisWrapper() as db:
                    db.add_container(
                        container=container,
                        name=name, 
                        ip=cnt_ip,
                        host_ip=host_ip,
                        local_port=local_port,
                        remote_port=port)
            else:
                utils.print_message('| {} | \tStep: Creating container  Error: 1001'.format(name))
                self.destroy_container(name=name)
                return ret
            if self.__exec_cmd(name=name, iso2=iso2):
                utils.print_message('| {} | \tStep: Getting VPN External IP'.format(name))
                container = self._dc.get_container_info(name)
                vpn_external_ip = utils.get_vpn_external_ip(
                    container=container, external_ip=external_ip)
                utils.print_message('| {} | \tStep: Saving VPN External IP'.format(name))
                if vpn_external_ip:
                    with RedisWrapper() as db:
                        db.add_container_vp_ip(
                            container_name=name, value=vpn_external_ip)
                    ret = True
                else:
                    utils.print_message('| {} | Cant get external vpn ip'.format(name))
                    self.destroy_container(name=name)
            else:
                self.destroy_container(name=name)
        except Exception:
            pass
        finally:
            return ret

    def __exec_cmd(self, name: str, iso2: str = None) -> bool:
        ret = False
        try:
            utils.print_message('| {} | \tStep: Exec VPN'.format(name))
            _ = self._dc.exec_vpn(name, iso2)
            ret = True
        except Exception as e:
            print("run {}".format(utils.print_exception()))
            print(" {} | Error running VPN - Error:{}").format(name, e)
        return ret

    def destroy_container(self, name: str) -> bool:
        ret = False
        try:
            utils.print_message('| {} | \tStep: Destroying Container'.format(name))
            if self._dc.container_stop_remove(name):
                utils.print_message('| {} | \tStep: Deleting Data'.format(name))
                with RedisWrapper() as db:
                    utils.print_message('| {} | \tStep: Deleting Container'.format(name))
                    db.del_container(container_name=name)
                    utils.print_message('| {} | \tStep: Deleting VPN IP'.format(name))
                    db.del_container_vp_ip(container_name=name)
                    utils.print_message('| {} | \tStep: Getting AUTH File'.format(name))
                    # auth_file = db.get_container_auth(container_name=name)
                    utils.print_message('| {} | \tStep: Decresing AUTH'.format(name))
                    # vutils.decrease_auth(auth_file['vpn_name'], auth_file['auth_file'])
                    utils.print_message('| {} | \tStep: Deleting AUTH File'.format(name))
                    db.del_container_auth(container_name=name)
                ret = True
        except Exception as e:
            utils.print_message("| {} | Error destroying Container error:{}".format(
                name, e))
            # print("error")
        finally:
            return ret
        


#!/usr/bin/python3
# -*- coding: utf-8 -*-
import redis
from json import loads, dumps
from typing import List, Dict, Tuple, Set
from hashlib import sha256
from datetime import datetime
from time import sleep
from json import dumps
from collections import Counter
import erebo.settings.settings as settings
import erebo.core.utils as utils
from erebo.core.utils import get_enviroment

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"

class RedisWrapper(object):
    shared_state = {}

    def __init__(self):
        _pool_config = {
            "host": settings.settings['redis']['host'],
            "port": settings.settings['redis']['port'],
            "db": settings.settings['redis']['dbname']
        }
        _pass = settings.settings['redis'].get('passw', None)
        if _pass:
            sha_pass = sha256(_pass.encode('utf8')).hexdigest()
            _pool_config['password'] = sha_pass
        self.__conn_pool = redis.ConnectionPool(**_pool_config)
        self.__conn = redis.Redis(connection_pool=self.__conn_pool)
        self.__sc_enviroment = get_enviroment()['sc_enviroment']
        self.__pubsub = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self._RedisWrapper__pubsub:
            self._RedisWrapper__pubsub.unsubscribe()
        if self.__conn_pool:
            self.__conn_pool.disconnect()

    def __del__(self):
        if self._RedisWrapper__pubsub:
            self._RedisWrapper__pubsub.unsubscribe()
        pass

    def ping(self):
        return self.__conn.ping()

    def get_user(self, user: str) -> Dict:
        name = 'users_ws'
        ret = {}
        try:
            con = self.__conn
            rows = con.hget(name=name, key=user)
            ret = loads(rows.decode('utf8'))
        except Exception as e:
            print(e)
        finally:
            return ret

    def add_container(self, container: str, name: str, ip: str,
            local_port: str, remote_port: str, host_ip: str) -> bool:
        name = 'containers'
        ret = False
        data = {
            "id": container.id,
            "ip": ip,
            "host_ip": host_ip,
            "local_port": local_port,
            "remote_port": remote_port
        }
        try:
            con = self.__conn
            _ = con.hset(
                name=name,
                value=dumps(data),
                key=str(container.name)
            )
            ret = True
        except Exception as e:
            utils.print_message('| {} | ERROR adding container in Redis Err:{}'.format(name, e))
        finally:
            return ret

    def get_containers(self) -> Dict:
        name = 'containers'
        ret = []
        try:
            con = self.__conn
            rows = con.hgetall(name=name)
            for row in rows.items():
                row_details = loads(row[1].decode('utf8'))
                ret.append({
                    "name": row[0].decode('utf8'),
                    "id": row_details['id'],
                    "ip": row_details['ip']
                })
        except Exception as e:
            utils.print_message('| {} | ERROR agetting container from Redis Err:{}'.format(name, e))
        finally:
            return ret

    def add_container_auth(self, container_name: str, auth_file: str,
            vpn_name: str, vpn_file: str):
        name = 'containers_auth'
        ret = {}
        vpn_country = utils.get_location_from_file(vpn_name, vpn_file)
        value = {
            "auth_file": auth_file,
            "vpn_name": vpn_name,
            "vpn_file": vpn_file,
            "vpn_country": vpn_country.get('country', '')
        }
        try:
            con = self.__conn
            _ = con.hset(
                name=name,
                value=dumps(value),
                key=container_name
            )
        except Exception as e:
            utils.print_message('| {} | ERROR adding auth data to Redis Err:{}'.format(container_name, e))
        finally:
            return ret

    def get_container_auth(self, container_name: str):
        name = 'containers_auth'
        ret = {}
        try:
            con = self.__conn
            ret = con.hget(name=name, key=container_name)
            ret = loads(ret)
        except Exception as e:
            utils.print_message('| {} | ERROR getting auth data from Redis Err:{}'.format(container_name, e))
        finally:
            return ret

    def del_container_auth(self, container_name: str) -> bool:
        name = 'containers_auth'
        ret = False
        try:
            con = self.__conn
            _ = con.hdel(name, container_name)
            ret = True
        except Exception as e:
            utils.print_message('| {} | ERROR deleting auth data in Redis Err:{}'.format(container_name, e))
        finally:
            return ret

    def del_container(self, container_name: str):
        name = 'containers'
        ret = {}
        try:
            con = self.__conn
            _ = con.hdel(
                name,
                str(container_name),
            )
        except Exception as e:
            utils.print_message('| {} | ERROR deleting container from Redis Err:{}'.format(container_name, e))
        finally:
            return ret

    def add_to_route(self, ip: str, container_ip: str, position: int) -> bool:
        name = 'routes:{}'.format(ip)
        ret = False
        try:
            con = self.__conn
            _ = con.hset(name, key=container_ip, value=position)
        except Exception as e:
            print(e)
        finally:
            return ret

    def get_to_route(self, ip: str):
        name = 'routes:{}'.format(ip)
        ret = []
        try:
            con = self.__conn
            ret = [
                {
                    'ip': k.decode('utf-8'),
                    'position': v.decode('utf-8')}
                for k, v in  con.hgetall(name=name).items()]
        except Exception as e:
            print(e)
        finally:
            return ret

    def del_to_route(self, ip: str, container_ip: str) -> bool:
        name = 'routes:{}'.format(ip)
        ret = False
        try:
            con = self.__conn
            _ = con.hdel(name, container_ip)
        except Exception as e:
            print(e)
        finally:
            return ret

    def add_container_vp_ip(self, container_name: str, value: Dict) -> bool:
        name = 'containers_vpn_ip'
        ret = False
        try:
            con = self.__conn
            _ = con.hset(
                name=name,
                value=dumps(value),
                key=container_name
            )
            ret = True
        except Exception as e:
            utils.print_message('| {} | ERROR adding container ip in Redis Err:{}'.format(container_name, e))
        finally:
            return ret

    def get_container_vp_ip(self, container_name: str) -> bool:
        name = 'containers_vpn_ip'
        ret = False
        try:
            con = self.__conn
            _ = con.hget(name=name, key=container_name)
            ret = True
        except Exception as e:
            utils.print_message('| {} | ERROR getting container ip from Redis Err:{}'.format(container_name, e))
        finally:
            return loads(ret)

    def get_container_vp_name(self, ip: str) -> str:
        name = 'containers_vpn_ip'
        ret = False
        try:
            con = self.__conn
            ret = [
                k.decode('utf-8')
                for k, v in con.hgetall(name=name).items() 
                if loads(v.decode('utf-8'))['ip'] == ip][0]
        except Exception as e:
            utils.print_message('| {} | ERROR getting container name from Redis Err:{}'.format(ip, e))
        finally:
            return ret

    def del_container_vp_ip(self, container_name: str) -> bool:
        name = 'containers_vpn_ip'
        ret = False
        try:
            con = self.__conn
            _ = con.hdel(name, container_name)
            ret = True
        except Exception as e:
            utils.print_message('| {} | ERROR delteing container ip from Redis Err:{}'.format(container_name, e))
        finally:
            return ret

    def get_task(self, channel:str, callback, **kwargs):
        print("wait for message")
        pubsub = self.__conn.pubsub()
        pubsub.subscribe(channel)
        self.__pubsub = pubsub
        while True:
            msg = self.__pubsub.get_message()
            if msg:
                if msg.get('type', '') == 'message':
                    callback(msg=msg['data'].decode('utf8'), **kwargs)

    def get_pubsub(self, channel:str):
        pubsub = self.__conn.pubsub()
        pubsub.subscribe(channel)
        return pubsub

    def get_pub(self, channel: str, msg):
        ret = False
        try:
            self.__conn.publish(channel, msg)
            ret = True
        except Exception as e:
            print(e)
        return ret

    def send_response(self, channel: str, task:str, status: bool, msg: str):
        dct_msg =  {
            "task": task,
            "status": status,
            "msg": msg
        }
        msg = dumps(dct_msg)
        self.get_pub(channel=channel, msg = msg)

    def get_used_vpn(self):
        name = 'containers_auth'
        ret = False
        try:
            con = self.__conn
            rows = [
                loads(v.decode('utf-8')) 
                for k, v in con.hgetall(name).items()
            ]
            vpns_used = [
                (x["vpn_name"], x["auth_file"]) 
                for x in  rows
            ]
            ret = dict(Counter(vpns_used)) 
        except Exception as e:
            utils.print_message('| {} | ERROR deleting auth data in Redis Err:{}'.format('', e))
        finally:
            return ret

    def add_country_iso_list(self, country_iso_list: Dict) -> bool:
        name = 'country_iso2_list'
        ret = False
        try:
            con = self.__conn
            _ = self.__conn.delete(name)
            for k, v in country_iso_list.items():
                _ = con.hset(
                    name=name,
                    value=int(v),
                    key=str(k)
                )
            ret = True
        except Exception as e:
            utils.print_message('| ERROR fecth data to Redis Err:{}'.format(e))
        finally:
            return ret


#!/usr/bin/python3
# -*- coding: utf-8 -*-

errors = {
    "JSONDecodeError": {
        'code': 300,
        'msg': 'the body of request doesnt have json format'
    },
    "ValidationError": {
        'code': 302,
        'msg': 'error in the params of json object'
    },
    "DataError": {
        'code': 500,
        'msg': 'error in the data'
    },
    "ConectionError": {
        'code': 403,
        'msg': 'error in the conection'
    },
    "UnauthorizedError": {
        'code': 506,
        'msg': 'unauthorized login'
    },
    "UnauthorizedTokenError": {
        'code': 401,
        'msg': 'unauthorized token'
    },
    "ExpiredTokenError": {
        'code': 408,
        'msg': 'token has expired'
    },
    "NotAuthorizationTokenError": {
        'code': 402,
        'msg': 'missing authorization'
    },
    "ExternalError": {
        'code': 405,
        'msg': 'error connecting with external server'
    }
}


#!/usr/bin/python3
# -*- coding: utf-8 -*-
import argparse
from os import path
import sys
from shutil import copyfile

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"
__message__ = '''
This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.
'''


def _copy_files(config_file:str, auth_file:str) -> bool:
    ret = False
    try:
        APP_PATH = path.dirname(path.dirname(path.abspath(__file__)))
        dst_auth_file = f'{APP_PATH}/erebo/config/files_auth.json'
        dst_config_files = f'{APP_PATH}/erebo/config/main.ini'
        x = copyfile(config_file, dst_config_files)
        y = copyfile(auth_file, dst_auth_file)
        if x and y:
            ret = True
    except Exception as e:
        print(e)

    finally:
        return ret


def main_pypi():
    import sys
    print(f"{__message__}")
    print(f"Version PYPI {__version__}")
    config_file, auth_file = sys.argv[1], sys.argv[2]
    if _copy_files(config_file, auth_file):
        from erebo.controllers.daemon_io import Daemon
        dm = Daemon()
        dm.run()
    else:
        print("Error initializing Erebo")


def main(config_file:str, auth_file:str):
    if _copy_files(config_file, auth_file):
        from erebo.controllers.daemon_io import Daemon
        dm = Daemon()
        dm.run()
    else:
        print("Error initializing Erebo")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Erebo')
    parser.add_argument(
        "-c",
        "--config",
        dest='config_file',
        required=True,
        type=str,
        help="location of the configuration file")
    parser.add_argument(
        "-a",
        "--auth",
        dest='auth_file',
        required=True,
        type=str,
        help="location of the auth file")
    args = parser.parse_args()
    print("{}".format(__message__))
    print("Version {}".format(__version__))
    main(args.config_file, args.auth_file)


#!/usr/bin/python3
# -*- coding: utf-8 -*-
from concurrent.futures import ProcessPoolExecutor
from concurrent.futures import ThreadPoolExecutor
from json import loads
# import erebo.controllers.test_len as tl
import erebo.core.utils as utils
import erebo.settings.settings as settings
from erebo.model.rediss import RedisWrapper
from erebo.controllers.controller import Controller
from erebo.controllers.daemon import DaemonFnc

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"


class Daemon(object):
    """[summary]
    
    Arguments:
        object {[type]} -- [description]
    """

    def __init__(self):
        self.futures = []

    def _fnc_callback(self, msg:str):
        msg_dict = loads(msg)
        ct =  Controller()
        if msg_dict.get('task', None):
            ct.main(msg_dict)

    def get_message(self, callback, **kwargs):
        db = RedisWrapper()
        pubsub = db.get_pubsub(channel=settings.settings['redis']['channel'])
        while True:
            msg = pubsub.get_message()
            if msg:
                if msg.get('type', '') == 'message':
                    callback(msg=msg['data'].decode('utf8'), **kwargs)

    def proc_channel(self):
        while True:
            try:
                print("connecting to channel")
                self.get_message(callback=self._fnc_callback)
            except ConnectionError:
                print("PPL {}".format(utils.print_exception()))
                print("closed channel")
            except Exception as e:
                print("PPL {}".format(utils.print_exception()))
                print("Error Exited from PPL - Error:{}").format(e)
            
    # def get_message(self, callback, **kwargs):
    #     try:
    #         db = RedisWrapper()
    #         pubsub = db.get_pubsub(channel='test')
    #         while True:
    #             msg = pubsub.get_message()
    #             if msg:
    #                 if msg.get('type', '') == 'message':
    #                     callback(msg=msg['data'].decode('utf8'), **kwargs)
    #     except Exception:
    #         print("PPL {}".format(utils.print_exception()))
    #         print("Error Exited from PPL - Error:{}").format(e)

    # def proc_channel(self):
    #     self.get_message(callback=self._fnc_callback)

    
    def proc_test(self):
        pass
        # tl.run()

    def proc_daemon_fnc(self):
        dm = DaemonFnc()
        # dm.restart_vpn_time(time_seg=300)
        dm.restart_vpn_time(time_seg=0)

    def run(self):
        executor = ThreadPoolExecutor(max_workers=2)
        executor.submit(self.proc_channel)
        # executor.submit(self.proc_test)
        # executor.submit(self.proc_daemon_fnc)


#!/usr/bin/python3
# -*- coding: utf-8 -*-
from typing import List, Dict, Tuple, Set
import erebo.settings.settings as settings
import erebo.core.utils as utils
from erebo.model.rediss import RedisWrapper

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"

class Routes(object):
    """[summary]

    Arguments:
        object {[type]} -- [description]
    """
    def __init__(self, *args, **kwargs):
        pass

    def add_routes(self, ip: str, every_jump: int = 1) -> bool:
        ret = False
        try:
            # cnt = 1
            with RedisWrapper() as db:
                containers = db.get_containers()
            cnt = len(containers)
            ret_proc = utils.route_target(False)
            for container in containers:
                ret_proc = utils.to_route(ip, container['ip'], cnt, True, every_jump)
                if not ret_proc: 
                    raise ValueError
                with RedisWrapper() as db:
                    _ = db.add_to_route(ip=ip, container_ip=container['ip'],
                            position=cnt)
                cnt -= 1
            ret_proc = utils.route_target(True)
            if not ret_proc: 
                raise ValueError
            ret = True
        except Exception as e:
            print(e)
        return ret

    def del_routes(self, ip: str, every_jump: int = 1) -> bool:
        ret = False
        try:
            with RedisWrapper() as db:
                routes = db.get_to_route(ip=ip)
            for route in routes:
                ret_proc = utils.to_route(
                    ip, route['ip'], route['position'], False, every_jump)
                if not ret_proc: 
                    raise ValueError
                with RedisWrapper() as db:
                    _ = db.del_to_route(ip=ip, container_ip=route['ip'])
            ret = True
            if not ret_proc: 
                raise ValueError
        except Exception as e:
            print(e)
        return ret


#!/usr/bin/python3
# -*- coding: utf-8 -*-
# import sys

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2017, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"

# sys.path += __path__
# =====================================================
# init
# =====================================================


def init(cont):
    '''
    This member function does nothing. It is a hook
    to be called by the BGE to trigger the initialization.
    '''
    pass



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from configparser import ConfigParser
from json import load
from os import path, environ
# import sys
# from platform import node


settings = dict()
APP_PATH = path.dirname(path.dirname(path.abspath(__file__)))
ROOT_PATH = path.dirname(APP_PATH)


def basepath(*path_add):
    return path.join(APP_PATH, *path_add)


# **************************************************************
# DIRECTORY
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Sistema de archivos, rutas a directorios de la aplicación
#
# storage: Directorio principal de almacenamiento
# tmp: Directorio temporal
# install: Directorio con contenido de instalación
# dump: Ficheros dump en caso de errores criticos
# uploads: Directorio subida de ficheros
# vault: Directorio de certificados
# template: Directorio base de templates
# schema: Directorio de definición de schemas
# ------------------------------------------------------------//
settings['storage.path'] = basepath('storage')
settings['tmp.path'] = basepath('storage', 'tmp')
settings['install.path'] = basepath('storage', 'install')
settings['dump.path'] = basepath('storage', 'dump')
settings['uploads.path'] = basepath('storage', 'uploads')


# **************************************************************
# CONFIGURATION
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Archivos de configuración
#
# main: Fichero de configuración principal
# env.main: Fichero de configuración principal del entorno
# ------------------------------------------------------------//
# enviroments = load(open('erebo/config/enviroment.json'))
enviroments = load(open(f'{APP_PATH}/config/enviroment.json'))
file_name = enviroments.get(environ.get('sc_enviroment', ''), 'main.ini')

settings['config.file.main'] = basepath('config', file_name)
print('Using enviroment: {}'.format(settings['config.file.main']))
# settings['config.file.env'] = basepath('config', 'main.env.ini')
# Gestor de configuración
config = ConfigParser(allow_no_value=True)
config.read((settings['config.file.main'],))
# enviroment
settings['enviroment'] = environ.get('sc_enviroment', 'main')
# Redis
settings['redis'] = {}
settings['redis']['host'] = config.get('Redis', 'host')
settings['redis']['port'] = config.get('Redis', 'port')
settings['redis']['passw'] = config.get('Redis', 'passw')
settings['redis']['dbname'] = config.get('Redis', 'dbname')
settings['redis']['channel'] = config.get('Redis', 'channel')
settings['redis']['channel_response'] = config.get('Redis', 'channel_response')
# file
settings['ovpn_root_dir'] = config.get('Files', 'ovpn_root_dir')
# sudo
settings['sudo_user'] = config.get('Sudo', 'user')
settings['sudo_pass'] = config.get('Sudo', 'passw')


#!/usr/bin/python3
# -*- coding: utf-8 -*-
from typing import List, Dict, Tuple, Set
from erebo.core.docker_project import DockerProject
from erebo.core.utils import ping_vpn


__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"


class CheckVpn(object):

    def __init__(self, *args, **kwargs):
        self.__docker_client = DockerProject()
        # return super().__init__(*args, **kwargs)

    def __get_containers_info(self, all: bool = False):
        return  [
            {
                "container": x,
                "name": x.name, 
                "status": x.status, 
                "ip_addrs": x.attrs['NetworkSettings']['IPAddress'],
                "ports": x.attrs['NetworkSettings']['Ports'],
                "port": list(x.attrs['NetworkSettings']['Ports'])[0].split('/')[0]
            } 
            for x in self.__docker_client.get_containers(False)]

    def test_status(self):
        ret = []
        list_containers = self.__get_containers_info()
        for container in list_containers:
            status = ping_vpn(container["ip_addrs"]+':'+container["port"])
            ret.append(
                {
                    "container": container['container'],
                    "status": status['code']
                })
        return ret
            



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from datetime import datetime
from typing import List, Dict, Tuple, Set
import erebo.core.utils as utils
import erebo.settings.settings as settings
from erebo.controllers.up_container import UpContainer
from erebo.controllers.routes import Routes
from erebo.controllers.restart_vpn import RestartVpn
from erebo.model.rediss import RedisWrapper

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"


class Dispacher(object):
    """Class Dispacher """

    upc = UpContainer()
    route = Routes()
    rvpn =  RestartVpn()
    def __init__(self):
        pass    

    @classmethod
    def create(cls, *args, **kwargs):
        ret = False
        try:
            upc = cls.upc
            name = kwargs['name']
            iso2 = kwargs.get('iso2', None)
            utils.print_message('| {} | Creating container'.format(name))
            ret = upc.create_container(name=name, iso2=iso2)
            if ret:
                utils.print_message('| {} | Created container'.format(name))
            else:
                utils.print_message('| {} | Cant\'n create container'.format(name))
        except Exception as e:
            utils.print_message("| {} | Error creating container Error: {}".format(name, e))            
        finally:
            cls._send_response(kwargs['task'], ret, kwargs)

    @classmethod
    def destroy(cls, *args, **kwargs):
        ret = False
        try:
            upc = cls.upc
            name = kwargs['name']
            utils.print_message('| {} | Destroying container'.format(name))
            ret = upc.destroy_container(name=name)
            if ret:
                utils.print_message('| {} | Destroyed container'.format(name))
            else:
                utils.print_message('| {} | Cant\'n destroy container'.format(name))
        except Exception as e:
             utils.print_message(
                 "| {} | Error destroying container Error: {}".format(e, name)) 
        finally:
            cls._send_response(kwargs['task'], ret, kwargs)

    @classmethod
    def add_route(cls, *args, **kwargs):
        ret = False
        try:
            route = cls.route
            ip = kwargs['ip']
            every_jump = int(kwargs.get('every_jump', 1))
            utils.print_message('| {0} | Add IP:{0} to Route'.format(ip))
            ret = route.add_routes(ip=ip, every_jump=every_jump)
            if ret:
                utils.print_message('| {0} | Added IP:{0} to Route'.format(ip))
            else:
                utils.print_message('| {0} | Cant\'n add IP:{0} to Route'.format(ip))
        except Exception as e:
            print(" {} | Error adding route Error: {}".format(e, ip))
        finally:
            cls._send_response(kwargs['task'], ret, kwargs)

    @classmethod
    def del_route(cls, *args, **kwargs):
        ret = False
        try:
            route = cls.route
            ip = kwargs['ip']
            every_jump = int(kwargs.get('every_jump', 1))
            utils.print_message('| {0} | Delete IP:{0} to Route'.format(ip))
            ret = route.del_routes(ip=ip, every_jump=every_jump)
            if ret:
                utils.print_message('| {0} | Deleted IP:{0} to Route'.format(ip))
            else:
                utils.print_message('| {0} | Cant\'n delete IP:{0} to Route'.format(ip))
        except Exception as e:
            print(" {} | Error deleting route Error: {}".format(e, ip))
        finally:
            cls._send_response(kwargs['task'], ret, kwargs)

    @classmethod
    def restart_vpn(cls, *args, **kwargs):
        ret = False
        try:
            container_name = kwargs['name']
            iso2 = kwargs.get('iso2', None)
            utils.print_message('| {} | Restart VPN in container'.format(container_name))
            try:
                ret = cls.rvpn.run(container_name=container_name, iso2=iso2)
            except Exception as e:
                print(" {} | Error executing vpn Error: {}".format(e, container_name))
            if ret:
                utils.print_message('| {} | Restarted VPN in container'.format(container_name))
            else:
                utils.print_message('| {} |  Cant\'n restart VPN in container'.format(container_name))
        except Exception as e:
            print(" {} | Error restarting vpn Error: {}".format(e, container_name)) 
        finally:
            cls._send_response(kwargs['task'], ret, kwargs)

    @classmethod
    def restart_vpn_ip(cls, *args, **kwargs):
        ret = False
        try:
            ip = kwargs['ip']
            iso2 = kwargs.get('iso2', None)
            with RedisWrapper() as db:
                container_name = db.get_container_vp_name(ip=ip)
            utils.print_message('| {} | Restarted VPN IP in container'.format(container_name))
            try:
                ret = cls.rvpn.run(container_name=container_name, iso2=iso2)
            except Exception as e:
                print(" {} | Error executing vpn Error: {}".format(e, container_name))
            if ret:
                utils.print_message('| {} | Restarted VPN IP in container'.format(container_name))
            else:
                utils.print_message('| {} | Cant\'n restart VPN IP in container'.format(container_name))
        except Exception as e:
            print(" {} | Error restarting vpn ip Error: {}".format(e, ip)) 
        finally:
            cls._send_response(kwargs['task'], ret, kwargs)
    
    @classmethod
    def _send_response(cls, task, ret, kwargs):
        with RedisWrapper() as db:
            db.send_response(
                settings.settings['redis']['channel_response'],
                task, ret, kwargs
                )


#!/usr/bin/python3
# -*- coding: utf-8 -*-
import re
import fnmatch
from json import load, loads, dump
from os import environ, listdir, walk, path
from typing import List, Dict, Tuple, Set
import erebo.settings.settings as settings
import erebo.core.utils as utils
from erebo.model.rediss import RedisWrapper

def _list_files (directory: str) -> Dict:
    """Return ovpn data with a list of files in key filenames
    
    Arguments:
        directory {str} -- dir path
    
    Returns:
        Dict -- ovpn file data with a list of ovpn files
    """
    if directory[-1] == '/':
        directory = directory[:-1]
    includes = ['*.ovpn']
    includes = r'|'.join([fnmatch.translate(x) for x in includes])
    _, _, filenames = [val for val in walk(directory)][0]
    files = [
        fname
        for fname in filenames
        if re.match(includes, fname)
    ]
    return files

def _find_vpn_country(config: List, files: List, iso2: str) -> List:
    vpn_files = []
    countries = utils.load_countries()
    country = countries[iso2]
    country_name = country['country'].strip().lower()
    country_iso3 = country['iso3'].strip().lower()
    country_iso2 = iso2.strip().lower()
    cp = re.compile(config[0], re.IGNORECASE)
    for i in files:
        try:
            ret = re.search(cp, i).group(1) 
            ret = ret.replace('-', '').replace('_', ' ').strip().lower()
            if ret == country_iso2:
                vpn_files.append(i)
            elif ret == country_iso3:
                vpn_files.append(i)
            elif ret in country_name:
                vpn_files.append(i)
        except Exception as e:
            print(e)
    return vpn_files

def get_list_countries_iso2() -> Dict:
    files_auth = utils.load_vpns_config()
    # files_auth = load(open('config/files_auth.json'))
    countries = utils.load_countries()
    providers = [(k, v['directory']) for k, v in files_auth.items()]
    ret = {}
    for provider in providers:
        provider_name = provider[0]
        files = _list_files(provider[1])
        config = files_auth[provider_name]['regex_location']
        provider_iso2 = {}
        for iso2, _ in countries.items():
            cnt_files = len(_find_vpn_country(config, files, iso2))
            if cnt_files > 0:
                provider_iso2[iso2] = cnt_files
        ret[provider_name] = provider_iso2
    return ret
            
def get_list_countries_iso2_total() -> Dict:
    list_countries = get_list_countries_iso2()
    list_iso2 = [] 
    for _, v in list_countries.items(): 
        list_iso2 += list(v.keys()) 
    set_iso2 = set(list_iso2)  
    ret = {} 
    for iso2 in set_iso2: 
        cnt = 0 
        for _,v in list_countries.items(): 
            cnt += v.get(iso2, 0) 
        ret[iso2] = cnt
    return ret

def main():
    lst_countries = get_list_countries_iso2_total()
    with RedisWrapper() as db:
        _ = db.add_country_iso_list(lst_countries)

if __name__ == "__main__":
    print("Updating country list in Redis")
    main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from typing import List, Dict, Tuple, Set
from asyncio import sleep
from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import as_completed
from erebo.dispachers.dispacher import Dispacher as ds

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"


class Controller(object):
    """Class Dispacher """

    def __init__(self):
        self.futures = []
        self.executor = ThreadPoolExecutor(max_workers=1)

    def main(self, msg: Dict):
        executor = self.executor
        task = msg['task']
        if task == 'create_container':
            self.futures.append(
                executor.submit(ds.create, **msg))
        elif task == 'destroy_container':
            self.futures.append(
                executor.submit(ds.destroy, **msg))
        elif task == 'add_routes':
            self.futures.append(
                executor.submit(ds.add_route, **msg))
        elif task == 'del_routes':
            self.futures.append(
                executor.submit(ds.del_route, **msg))
        elif task == 'restart_vpn':
            self.futures.append(
                executor.submit(ds.restart_vpn, **msg))
        elif task == 'restart_vpn_ip':
            self.futures.append(
                executor.submit(ds.restart_vpn_ip, **msg))
        else:
            print(task)


#!/usr/bin/python3
# -*- coding: utf-8 -*-

from time import sleep
from datetime import datetime
from erebo.model.rediss import RedisWrapper
from erebo.controllers.restart_vpn import RestartVpn
import erebo.core.utils as utils

__author__ = "Lennin Caro"
__copyright__ = "Copyright 2019, The Erebo (VPN WAY) Project"
__credits__ = ["Lennin Caro"]
__license__ = "GPL"
__version__ = "1.1.23"
__maintainer__ = "Lennin Caro"
__email__ = "renjin25@gmail.com"
__status__ = "Production"

class DaemonFnc(object):
    """[summary]
    
    Arguments:
        object {[type]} -- [description]
    """

    def __init__(self):
        self.rvpn =  RestartVpn()

    def restart_vpn_time(self, time_seg: int):
        while True:
            sleep(time_seg)
            try:        
                with RedisWrapper() as db:
                    containers = db.get_containers()
                ret = False
                for container in containers:
                    container_name = container['name']
                    utils.print_message(
                        '| {} | Automatic restart VPN in container'.format(
                            container_name))
                    try:
                        ret = self.rvpn.run(container_name=container_name)
                    except Exception:
                        utils.print_message(
                            '| {} | Error running vpn Error'.format(
                                container_name))
                    if ret:
                        utils.print_message(
                            '| {} | Automatic restart VPN in done'.format(
                                container_name))
                    else:
                        utils.print_message(
                            '| {} | Cant\'n restart VPN in container'.format(
                                container_name))
            except Exception as e:
                print("restart_vpn_time {}".format(utils.print_exception()))


#!/usr/bin/python3
# -*- coding: utf-8 -*-

import random
import re
import fnmatch
from json import load, loads, dump
from os import environ, listdir, walk, path
from typing import List, Dict, Tuple, Set
from datetime import datetime
from erebo.model.rediss import RedisWrapper
import erebo.settings.settings as settings
import erebo.core.utils as utils


def get_auth_available(vpn_name: Tuple):
    files_auth = utils.load_vpns_config()
    # files_auth = load(open('config/files_auth.json'))
    ret = {
        "file": vpn_name[1],
        "vpn_name": vpn_name[0],
        "directory": files_auth[vpn_name[0]]['directory'],
        "container_dirpath": files_auth[vpn_name[0]]['container_dirpath']
    }
    if ret['directory'][-1] == '/':
        directory = ret['directory'][:-1]
    else:
        directory = ret['directory']
    ret['full_path'] = '/'.join((directory, vpn_name[1]))
    return ret

def list_files (directory: str) -> Dict:
    """Return ovpn data with a list of files in key filenames
    
    Arguments:
        directory {str} -- dir path
    
    Returns:
        Dict -- ovpn file data with a list of ovpn files
    """
    if directory[-1] == '/':
        directory = directory[:-1]
    includes = ['*.ovpn']
    includes = r'|'.join([fnmatch.translate(x) for x in includes])
    dirpath, dirnames, filenames = [val for val in walk(directory)][0]
    files = [
        fname
        for fname in filenames
        if re.match(includes, fname)
    ]
    ret = {
        "dirpath": dirpath,
        "dirnames": dirnames,
        "filenames": files
    }
    return ret

def get_ovpn_filename(directory: str) -> Dict:
    """Get a random file from list_files function
    
    Arguments:
        directory {str} -- dir path
    
    Returns:
        Dict -- ovpn file data
    """
    ret = {}
    filenames = list_files(directory)
    max_files = len(filenames['filenames'])
    if max_files > 0:
        rnd_number = random.randint(0, max_files - 1)
        filename = filenames['filenames'][rnd_number]
        ret = {
            "dirpath": filenames['dirpath'],
            "filename": filename,
            "full_path": "/".join((filenames['dirpath'], filename))
        }
    return ret

def get_ovpn_filename_iso(vpn_avaliable: Dict, iso2: str = None) -> Dict:
    """Get a random file from list_files function
    
    Arguments:
        directory {str} -- dir path
    
    Returns:
        Dict -- ovpn file data
    """
    ret = {}
    vpns = utils.load_vpns_config()
    # vpns = load(open('config/files_auth.json'))
    vpn_name = vpn_avaliable['vpn_name']
    vpn_location = vpns[vpn_name]['regex_location']
    vpn_files = list_files(vpns[vpn_name]['directory'])
    vpns_country = find_vpn_country(vpn_location, vpn_files['filenames'], iso2)
    max_files = len(vpns_country)
    if max_files > 0:
        rnd_number = random.randint(0, max_files - 1)
        filename = vpns_country[rnd_number]
        ret = {
            "dirpath": vpn_files['dirpath'],
            "filename": filename,
            "full_path": "/".join((vpn_files['dirpath'], filename))
        }
    return ret

def replace_auth_user(ovpn_file: str, auth_user_pass_file: str) -> bool:
    """Replaces the "auth-user-pass" line in the .ovpn file  adding the path of the
       file where the user and password are stored
    Arguments:
        ovpn_file {str} -- dir path to ovpn file
        auth_user_pass_file {str} -- dir path to file with user and pass
    
    Returns:
        bool -- True if could be replace
    """
    ret = False
    line_id = None
    try:
        with open(ovpn_file , 'r') as op: 
            lines = op.readlines()
        for idx, item in enumerate(lines): 
            if "auth-user-pass" in item: 
                line_id = idx    
                break
        if line_id:
            lines[line_id] = " ".join(("auth-user-pass", auth_user_pass_file)) + "\n"
            with open(ovpn_file , 'w') as op: 
                op.writelines(lines)
            ret = True
    except Exception as e:
        print(" {} | Error replacing auth user Error: {}".format(e, auth_user_pass_file))
    finally:
        return ret

def get_vpn_name(iso2: str = None) -> Tuple:
    vpns = utils.load_vpns_config()
    # vpns = load(open('config/files_auth.json')) 
    rows = [] 
    all_vpn = {}
    
    for vpn in vpns.keys(): 
        auths = [{  
            (vpn, k): v['max_connections']} 
            for k, v in vpns[vpn]['files_auth'].items()
            if v['max_connections'] > 0] 
        rows += auths
    for row in rows: 
        all_vpn = {**all_vpn, **row}
    with RedisWrapper() as db:
        used_vpn = db.get_used_vpn()
    avaliable_vpn = [key for key in all_vpn if all_vpn[key] - used_vpn.get(key,0) > 0]
    if iso2:
        vpn_iso = get_vpn_location(avaliable_vpn, iso2)
        avaliable_vpn = [x for x in avaliable_vpn if x[0] in vpn_iso] 
    return _get_random_vpn_name(avaliable_vpn)

def _get_random_vpn_name(vpns: List) -> Tuple:
    max_files = len(vpns)
    rnd_number = random.randint(0, max_files - 1)
    return vpns[rnd_number]

def decrease_auth(vpn_name: str, auth: str) -> bool:
    ret = True
    return ret

def get_vpn_location(avaliable_vpn: List, iso2: str):
    ret = {}
    vpns = utils.load_vpns_config()
    # vpns = load(open('config/files_auth.json'))
    aval_vpn = set(x[0] for x in avaliable_vpn)
    for vpn_name in aval_vpn:
        vpn_files = list_files(vpns[vpn_name]['directory'])
        vpn_location = vpns[vpn_name]['regex_location']
        ret[vpn_name] = find_vpn_country(vpn_location, vpn_files['filenames'], iso2)
    ret = [key for key in ret if len(ret[key]) > 0]
    return ret

def find_vpn_country(config: List, files: List, iso2: str):
    vpn_files = []
    # countries = load(open('config/countries.json', 'r'))
    countries = utils.load_countries()
    country = countries[iso2]
    country_name = country['country'].strip().lower()
    country_iso3 = country['iso3'].strip().lower()
    country_iso2 = iso2.strip().lower()
    cp = re.compile(config[0], re.IGNORECASE)
    for i in files:
        try:
            ret = re.search(cp, i).group(1) 
            ret = ret.replace('-', '').replace('_', ' ').strip().lower()
            if ret == country_iso2:
                vpn_files.append(i)
            elif ret == country_iso3:
                vpn_files.append(i)
            elif ret in country_name:
                vpn_files.append(i)
        except Exception as e:
            print(e)
    return vpn_files